/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
#ifndef __ERRORS_H__
#define __ERRORS_H__

#define EXIT_NORMAL 0
#define EXIT_MISSING_ARGUMENT -1
#define EXIT_BAD_HEADER_LINE -2
#define EXIT_NO_FILES -3
#define EXIT_FAILED_TO_CREATE_TMPFILES -4
#define EXIT_FAILED_TO_RESOLVE_HOST -5
#define EXIT_FAILED_TO_CREATE_SOCKET -6
#define EXIT_LOGON_FAILED -7
#define EXIT_POSTING_NOT_ALLOWED -8
#define EXIT_POSTING_FAILED -9
#define EXIT_UNKNOWN -10
#define EXIT_SOCKET_ERROR -11
#define EXIT_TOO_MANY_TEXT_FILES -12
#define EXIT_SIGNAL -13

#endif /* __ERRORS_H__ */
